/*
 * v1 Routes
 * -------------------------------------------------------------
 * Put some description here!
 */
const v1 = {

    /*
     * Define the prefix applied to each of these routes.
     * For example, if prefix is '/auth':
     *      '/' becomes '/auth'
     *      '/login' becomes '/auth/login'
     */
    prefix: '/dash/v1',

    /*
     * Define middleware that should be applied to all
     * routes defined in this file. Middleware should be
     * included using Flitter's global mw() function, but
     * it can also be added directly using require().
     */
    middleware: [
        // mw('Middleware Name'),
        _flitter.mw('auth:RequireAuth'),
        _flitter.mw('v1:Invite')
    ],

    /*
     * Define GET routes.
     * These routes are registered as GET methods.
     * Handlers for these routes should be specified as
     * an array of functions that are applied in order.
     *
     * mw() calls apply Flitter middleware
     * controller() calls get methods in Flitter controllers
     */
    get: {
        '/': [ _flitter.controller('dash:v1').main ],

        '/project/new': [ _flitter.controller('dash:v1').new_project_show ],
        '/project/view/:id': [ _flitter.controller('dash:v1').project_view ],
        '/project/delete/:id': [ _flitter.controller('dash:v1').project_delete_show ],
        '/project/edit/:id': [ _flitter.controller('dash:v1').project_edit_show ],
        '/:api/share/:id': [ _flitter.controller('dash:v1').project_share_show ],
        '/:api/share/:id/share/:user': [ _flitter.controller('dash:v1').project_share_do ],
        '/:api/share/:id/share/:user/edit': [ _flitter.controller('dash:v1').project_share_edit_do ],
        '/:api/share/:id/revoke/:user': [ _flitter.controller('dash:v1').project_share_revoke ],
        '/:api/share/:id/revoke/:user/edit': [ _flitter.controller('dash:v1').project_share_revoke_edit ],
        '/:api/share/:id/transfer/:user': [ _flitter.controller('dash:v1').project_share_transfer ],
        '/:api/share/:id/invite': [ _flitter.controller('dash:v1').project_share_invite ],
        '/project/snippet/:id/new': [ _flitter.controller('dash:v1').project_snippet_new ],
        '/project/snippet/:id/view/:snippet': [ _flitter.controller('dash:v1').project_snippet_view ],
        '/project/snippet/:id/delete/:snippet': [ _flitter.controller('dash:v1').project_snippet_delete ],
        '/project/snippet/:id/edit/:snippet': [ _flitter.controller('dash:v1').project_snippet_edit ],

        '/using_devbug/:page': [ _flitter.controller('dash:v1').show_usage_page ],

        '/out/view/:id': [ _flitter.controller('dash:v1').out_view ],
        '/out/delete/:id/:project': [ _flitter.controller('dash:v1').out_delete ],

        '/code': [ _flitter.controller('dash:v1').view_code ],

        '/invitation/accept': [ _flitter.controller('dash:v1').accept_invite ],
    },

    /*
     * Define POST routes.
     * These routes are registered as POST methods.
     * Handlers for these routes should be specified as
     * an array of functions that are applied in order.
     *
     * mw() calls apply Flitter middleware
     * controller() calls get methods in Flitter controllers
     */
    post: {
        '/project/new': [ _flitter.controller('dash:v1').new_project_do ],
        '/project/delete/:id': [ _flitter.controller('dash:v1').project_delete_do ],
        '/project/edit/:id': [ _flitter.controller('dash:v1').project_edit_do ],
        '/project/snippet/:id/new': [ _flitter.controller('dash:v1').project_snippet_new_do ],
        '/project/snippet/:id/edit/:snippet': [ _flitter.controller('dash:v1').project_snippet_edit_do ],
    },
}

module.exports = v1
